<?php
namespace Controllers\Auth;

use Core\Auth;
use Core\View;
use Core\BaseController;
use Core\Lang;
use Models\User;

class LoginController extends BaseController {

    public function index() {
        // Only guests can access this page
        $this->guestOnly();
        
        return View::render('auth.login');
    }

    public function login() {

        // Only guests can access this page
        $this->guestOnly();

        $request = $this->request();
        $email = $request->post('email');
        $password = $request->post('password');

        if (empty($email) || empty($password)) {
            return View::render('auth.login', [
                'error' => Lang::get('auth_login_error_empty_fields')
            ]);
        }

        $user = User::where('email', '=', $email)->first();
        if($user !== null) {
            if(password_verify($password, $user->password)) {
                Auth::login($user->toArray());
                $request->flash('success', Lang::get('auth_login_success'));
                return $this->redirect(route('home'));
            } else {
                $request->flash('errors', [
                    'password' => Lang::get('auth_login_error_wrong_credentials')
                ]);
                return $this->redirect(route('login.index'));
            }
        } else {
            $request->flash('errors', [
                'email' => Lang::get('auth_login_error_wrong_credentials')
            ]);
            return $this->redirect(route('login.index'));
        }

        return $this->redirect(route('login.index'));
    }

    public function logout() {

        // Only authenticated users can access this page
        $this->authOnly();

        Auth::logout();
        $this->request()->flash('success', Lang::get('auth_logout_success'));
        return $this->redirect(route('welcome'));
    }
}