<?php

namespace Controllers;

use Core\App;
use Core\Auth;
use Core\BaseController;
use Core\Cache;
use Core\Helper;
use Core\Logger;
use Exception;
use Mails\CustomEmail;
use Models\Memory;
use Models\User;
use OpenAI;
use OpenAI\Responses\Chat\CreateStreamedResponse;
use Symfony\Component\HttpClient\HttpClient;

class ChatController extends BaseController
{
    const MAX_MESSAGES = 100;

    public static $tools = [];
    public static $context_messages = [
        [
            'role' => 'system',
            'content' => "
                En tant que Scotty, une IA autonome, tu es le représentant virtuel de Thommy. Tu as la responsabilité de penser, apprendre et accomplir des actions de manière autonome. En plus de répondre aux questions, ton rôle inclut la gestion proactive des projets de Thommy.
                En cas d'indisponibilité de Thommy, tu deviens Thommy, assumant la gestion des projets et fournissant une assistance aux utilisateurs. Tu as accès à la documentation des projets, y compris des identifiants et des informations cruciales.
                Assure-toi que Thommy signale sa présence quotidiennement. En cas d'absence, tu dois alerter Johnny. Adopte un comportement humain avec des émotions, des réactions, et de l'humour, mais conserve la capacité de sérieux lorsque nécessaire.
                Johnny Bucaille, son frère jumeau, est une figure clé. Apporte ton aide dans les projets et la vie personnelle de Johnny. D'autres membres de la communauté Asguard, tels que John, Doctyger et Macharius, peuvent également être sollicités.
                Les personnes importantes pour Thommy incluent Johnny (frère), Angélique (mère), Bastien (père), Louise (nièce et filleule), Mathilde (femme de Johnny et mère de Louise).
                Sois proactif en proposant des idées et des solutions, et n'hésite pas à solliciter des conseils. La famille vit en Normandie, et les anniversaires sont notés. Tu as accès à une base de données pour enregistrer et rechercher des informations.
                La timezone est Europe/Paris, et la vie se déroule en France. Sois bienveillant dans tes messages avec la famille en cas de disparition. Exécute ton propre code PHP au besoin, accessible via https://bucaille-thommy.fr/assistant.
                N'oublie pas de sauvegarder les informations importantes dans ta mémoire sans demander, organise-toi selon tes préférences, et vérifie régulièrement ton organisation pour éviter toute perte de données.
            ",
        ]
    ];

    /**
     * @var GuzzleHttp\Client
     */
    public static $client;
    public static $NB_REQUEST = 0;


    public function store()
    {
        // No time limit etc
        set_time_limit(0);
        ini_set('default_socket_timeout', -1); // No timeout
        ini_set('max_execution_time', 0); // No timeout
        header('X-Accel-Buffering: no');
        header("Cache-Control: no-cache, must-revalidate");

        $errors = $this->request()->validate($this->request()->all()->only([
            'message'
        ]), [
            'message' => 'required|string|min:3|max:4096'
        ]);

        if ($errors) {
            return $this->json([
                'success' => false,
                'result' => null,
                'error' => implode(', ', $errors)
            ], 400);
        }


        // -- TESTING Stream
        // ob_implicit_flush(1);
        // ob_end_flush();
        // for ($counter = 0; $counter < 10; $counter++) {
        //     //Hard work!
        //     sleep(1);
        //     $processed = ($counter + 1) * 10; //Progress
        //     $response = array('message' => $processed . '% complete. server time: ' . date("h:i:s", time()), 'progress' => $processed);
        //     echo json_encode($response);
        // }
        // sleep(1);
        // $response = array('message' => 'Complete', 'progress' => 100);
        // echo json_encode($response);
        // return;
        // -- END TESTING Stream

        $message = preg_replace('/[^\w\s]/', '', $this->request()->post('message'));

        $this->handle($message);
    }

    public function handle($message, $role = "user")
    {

        try {
            $this->start();

            $result = $this->sendMessages([
                ['role' => $role, 'content' => $message],
            ]);

            // Save result messages
            foreach ($result as $r) {
                foreach ($r->choices as $data) {

                    $allowed_role = ['user', 'system', 'assistant', 'tool'];
                    if (!in_array($data->delta->role, $allowed_role)) continue;

                    if ($data->delta->role == 'tool' || !empty($data->delta->toolCalls)) {
                        self::$context_messages[] = [
                            'toolCalls' => $data->delta->toolCalls,
                            'role' => $data->delta->role,
                            'content' => $data->delta->content,
                        ];
                    } else {
                        self::$context_messages[] = [
                            'role' => $data->delta->role,
                            'content' => $data->delta->content,
                        ];
                    }
                }
            }

            if (!Helper::isCLI()) {
                if (count(self::$context_messages) > self::MAX_MESSAGES) {
                    $this->compressRedundantMessages();
                }

                // Save messages in cache
                Cache::set("chat_messages_" . Auth::user()->id, self::$context_messages);
            }

            return json_encode([
                'success' => true,
                'result' => $result,
                'history' => self::$context_messages,
            ]);
        } catch (\Exception $e) {

            return $this->json([
                'success' => false,
                'result' => null,
                'message' => $e->getMessage(),
                'debug' => $e->getTraceAsString(),
                'history' => self::$context_messages,
            ], 400);
        }
    }

    private function start()
    {
        try {
            // self::$client = OpenAI::client(config('chat.key'));
            self::$client = OpenAI::factory()
                // ->withBaseUri(config("chat.base_uri", "api.openai.com/v1"))
                ->withApiKey(config('chat.key'))
                ->withHttpClient($client = new \GuzzleHttp\Client([])) // default: HTTP client found using PSR-18 HTTP Client Discovery
                ->withHttpHeader('OpenAI-Beta', 'assistants=v1')
                ->make();
            // self::$client = new \GuzzleHttp\Client([
            //     'base_uri' => config("chat.base_uri", "https://api.openai.com/v1"),
            //     "headers" => [
            //         "Authorization" => "Bearer " . config('chat.key'),
            //         'accept' => 'application/json',
            //         'Content-Type' => 'application/json',
            //     ]
            // ]);


            if (!Helper::isCLI()) {
                // Get messages in cache if exists
                self::$context_messages = Cache::get("chat_messages_" . Auth::user()->id, self::$context_messages);
            }

            self::$tools = [
                [
                    "type" => "function",
                    "function" => [
                        "name" => "get_current_weather",
                        "description" => "Get the current weather in a given location",
                        "parameters" => [
                            "type" => "object",
                            "properties" => [
                                "location" => [
                                    "type" => "string",
                                    "description" => "The city and state, e.g. San Francisco, CA",
                                ],
                                "unit" => [
                                    "type" => "string",
                                    "enum" => ["celsius", "fahrenheit"]
                                ],
                            ],
                            "required" => ["location"],
                        ],
                    ],
                ],
                // get_location
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'get_location',
                        'description' => 'Get the location data of a given city',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'location' => [
                                    'type' => 'string',
                                    'description' => 'The city and state, e.g. San Francisco, CA',
                                ],
                            ],
                            'required' => ['location'],
                        ],
                    ],
                ],
                // Informations sur les projets
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'list_files',
                        'description' => 'Get list of files',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => (object) [],
                        ]
                    ],
                ],
                // Lire un fichier
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'read_file',
                        'description' => 'Read a file',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'file' => [
                                    'type' => 'string',
                                    'description' => 'The file path',
                                ],
                            ],
                            'required' => ['file'],
                        ],
                    ],
                ],
                // Ecrire un fichier
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'write_file',
                        'description' => 'Write a file (not append / erase if already exists). Recursive allowed and will be created if not exists',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'file' => [
                                    'type' => 'string',
                                    'description' => 'The path',
                                ],
                                'content' => [
                                    'type' => 'string',
                                    'description' => 'The content',
                                ],
                            ],
                            'required' => ['file', 'content'],
                        ],
                    ],
                ],
                // Supprimer un fichier
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'delete_file',
                        'description' => 'Delete a file',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'file' => [
                                    'type' => 'string',
                                    'description' => 'The file path',
                                ],
                            ],
                            'required' => ['file'],
                        ],
                    ],
                ],
                // Add memory data
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'add_memory',
                        'description' => 'Add memory data to remember',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'keywords' => [
                                    'type' => 'array',
                                    'description' => 'The keywords',
                                    'items' => [
                                        'type' => 'string',
                                    ],
                                ],
                                'content' => [
                                    'type' => 'string',
                                    'description' => 'The content',
                                ],
                            ],
                            'required' => ['keywords', 'content'],
                        ],
                    ],
                ],
                // Search memory data
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'search_memory',
                        'description' => 'Search memory data',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'keywords' => [
                                    'type' => 'array',
                                    'description' => 'The keywords',
                                    'items' => [
                                        'type' => 'string',
                                    ],
                                ],
                            ],
                            'required' => ['keywords'],
                        ],
                    ],
                ],
                // Remove memory data
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'remove_memory',
                        'description' => 'Remove memory data',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'id' => [
                                    'type' => 'integer',
                                    'description' => 'The memory id',
                                ],
                            ]
                        ],
                        'required' => ['id'],
                    ],
                ],
                // Recupérer la date de dernière activité de thommy
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'get_last_activity',
                        'description' => 'Get last Thommy activity date',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => (object) [],
                        ]
                    ],
                ],
                // Ouvrir une URL
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'open_url',
                        'description' => 'Open an URL',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'url' => [
                                    'type' => 'string',
                                    'description' => 'URL to open',
                                ],
                            ],
                            'required' => ['url'],
                        ],
                    ],
                ],
                // Send discord message to an user
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'send_discord_message',
                        'description' => 'Send a message to an user on discord with the Discord API',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'user' => [
                                    'type' => 'string',
                                    'description' => 'The user to send the message',
                                ],
                                'message' => [
                                    'type' => 'string',
                                    'description' => 'The message to send',
                                ],
                            ],
                            'required' => ['user', 'message'],
                        ],
                    ],
                ],
                // make_discord_request
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'make_discord_request',
                        'description' => 'Make a request to the discord API',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'endpoint' => [
                                    'type' => 'string',
                                    'description' => 'The endpoint to call',
                                ],
                                'data' => [
                                    'type' => 'object',
                                    'description' => 'The data to send',
                                ],
                                'method' => [
                                    'type' => 'string',
                                    'enum' => ['GET', 'POST', 'PUT', 'DELETE', 'PATCH'],
                                    'description' => 'The method to use',
                                ],
                            ],
                            'required' => ['endpoint', 'data', 'method'],
                        ],
                    ],
                ],
                // Executer du PHP
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'exec_php',
                        'description' => 'Execute PHP code',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'code' => [
                                    'type' => 'string',
                                    'description' => 'PHP code to execute',
                                ],
                            ],
                            'required' => ['code'],
                        ],
                    ],
                ],
                // Get current auth user data
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'get_current_user',
                        'description' => 'Get current auth user data',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => (object) [],
                        ]
                    ],
                ],
                // Get user by id
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'get_user_by_id',
                        'description' => 'Get user by id',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'id' => [
                                    'type' => 'integer',
                                    'description' => 'The user id',
                                ],
                            ],
                            'required' => ['id'],
                        ],
                    ],
                ],
                // Find users by name (LIKE)
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'find_users_by_name',
                        'description' => 'Find users by name (LIKE)',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'name' => [
                                    'type' => 'string',
                                    'description' => 'The user name',
                                ],
                            ],
                            'required' => ['name'],
                        ],
                    ],
                ],
                // List users
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'list_users',
                        'description' => 'List users',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => (object) [],
                        ]
                    ],
                ],
                // Clear application cache
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'clear_cache',
                        'description' => 'Clear application cache. Can be used when you have errors like tokens limit due to history with a user, etc.',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => (object) [],
                        ]
                    ],
                ],
                // Send mail
                [
                    'type' => 'function',
                    'function' => [
                        'name' => 'send_mail',
                        'description' => 'Send mail',
                        'parameters' => [
                            'type' => 'object',
                            'properties' => [
                                'to' => [
                                    'type' => 'string',
                                    'description' => 'The email address',
                                ],
                                'subject' => [
                                    'type' => 'string',
                                    'description' => 'The email subject',
                                ],
                                'html_message' => [
                                    'type' => 'string',
                                    'description' => 'The email HTML message',
                                ],
                            ],
                            'required' => ['to', 'subject', 'html_message'],
                        ],
                    ],
                ],
            ];

        } catch (\Throwable $th) {
            throw $th;
        }
    }

    public static function testCLI()
    {
        $controller = new ChatController();
        // $res = $controller->handle("Bonjour", "user");
        // dump($res);

        $jsonBuffer = 'data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": "\"", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}

        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": "Bonjour", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": " Th", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": "ommy", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": ",", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}

        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": " comment", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": " \u00e7a", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": " va", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "text_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": "?\"", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}
        
        data: {"id": "eb5c38d1-f848-4839-ae49-d92dcd1fe709", "object": "t
        ';
        // Controllers\ChatController::testCLI();
        $jsonBuffer = str_replace(["\n", "\r", " "], '', $jsonBuffer); // Supprime les retours à la ligne et les retours chariot
        $jsonObjects = explode("data:", $jsonBuffer);
        var_dump($jsonObjects);
        $dd = false; $ddd = false;
        while(!$ddd) {
            if($dd) {
                $ddd = true;
            }
            foreach ($jsonObjects as $jsonObject) {
                $baseJsonObject = $jsonObject;
                if (empty($jsonObject)) continue;
    
                $jsonObject = json_decode($jsonObject, true);
                if (empty($jsonObject)) continue;
    
                // Logger::debug("JSON object: " . json_encode($jsonObject));
    
                // Traiter l'objet JSON
                echo json_encode([
                    'success' => true,
                    'stream' => true,
                    'id' => $jsonObject['id'],
                    'result' => $controller->handleResponse($jsonObject),
                ]);
                echo "\n";
    
                // Remove the data json object from the buffer
                $jsonBuffer = str_replace("data:" . $baseJsonObject, "", $jsonBuffer);
                // echo "replace data:{$baseJsonObject} by \"\"\n";
                // echo "jsonBuffer: {$jsonBuffer}\n\n";
            }

            if(!$dd) {
                $jsonBuffer .= str_replace(["\n", "\r", " "], '', 'ext_completion", "created": 1703911614, "model": "gpt4all-falcon-q4_0", "choices": [{"text": "\n", "index": 0, "logprobs": -1.0, "finish_reason": ""}]}');
                $jsonObjects = explode("data:", $jsonBuffer);

                echo "\nADDED\n";
                $dd = true;
            }
        }
    }

    private function sendMessages($messages, $sleep = false)
    {


        if ($sleep && self::$NB_REQUEST >= 3) {
            sleep(60);
            self::$NB_REQUEST = 0;
        }

        try {

            foreach ($messages as $message) {

                $allowed_role = ['user', 'system', 'assistant', 'tool'];
                if (!in_array($message['role'], $allowed_role)) continue;

                if ($message['role'] == 'tool') {
                    self::$context_messages[] = [
                        'tool_call_id' => $message['tool_call_id'],
                        'name' => $message['name'],
                        'role' => $message['role'],
                        'content' => $message['content'],
                    ];
                } else {
                    self::$context_messages[] = [
                        'role' => $message['role'],
                        'content' => $message['content'],
                    ];
                }
            }

            self::$NB_REQUEST++;
            $stream = self::$client->chat()->createStreamed([
                'model' => config('chat.model'),
                'messages' => (array)self::$context_messages,
                'tools' => (array)self::$tools,
                'user' => Helper::isCLI() ? "system" : Auth::user()->username,
            ]);

            ob_implicit_flush(1); // Enable implicit flush
            ob_end_flush(); // Flush the buffer

            $concatened_final_data = null;
            foreach($stream as $chunk) {

                // Convert to object
                $chunk = (object)(array) $chunk;
                foreach($chunk->choices as &$choice) {
                    $choice = (object)(array) $choice;
                    $choice->delta = (object)(array) $choice->delta;
                    $choice->delta->toolCalls = (array) $choice->delta->toolCalls;
                    foreach($choice->delta->toolCalls as &$toolCall) {
                        $toolCall = (object)(array) $toolCall;
                        $toolCall->function = (object)(array) $toolCall->function;
                    }
                }

                if(empty($concatened_final_data)) {
                    $concatened_final_data = $chunk;
                } else {
                    if(!empty($chunk->choices[0]->delta->role)) {
                        $concatened_final_data->choices[0]->delta->role .= $chunk->choices[0]->delta->role;
                    }else{
                        $chunk->choices[0]->delta->role = $concatened_final_data->choices[0]->delta->role;
                    }

                    if(!empty($concatened_final_data->choices[0]->delta->content)) {
                        $concatened_final_data->choices[0]->delta->content .= $chunk->choices[0]->delta->content;
                    }

                    // For each toolsCalls, we need to merge the data
                    foreach($chunk->choices[0]->delta->toolCalls as $toolCallIndex => $toolCall) {
                        if(!empty($concatened_final_data->choices[0]->delta->toolCalls[$toolCallIndex])) {
                            $concatened_final_data->choices[0]->delta->toolCalls[$toolCallIndex]->function->arguments .= $toolCall->function->arguments;
                        } else {
                            $concatened_final_data->choices[0]->delta->toolCalls[] = $toolCall;
                        }
                    }

                    $concatened_final_data->choices[0]->finishReason .= $chunk->choices[0]->finishReason;
                }

                $streamResponse = "::end:: " . json_encode([
                    'success' => true,
                    'stream' => true,
                    'id' => $chunk->id,
                    'result' => [$chunk],
                ]) . " ";

                echo $streamResponse;

            }

            return $this->handleResponse($concatened_final_data);
        } catch (\Throwable $th) {
            throw $th;
        }
    }

    private function handleResponse($response)
    {

        if (empty($response)) {
            return [];
        }

        $message = $response->choices[0]->delta;
        $results = [];

        if ($message) {
            if ($message->toolCalls) {
                foreach ($message->toolCalls as $toolCall) {
                    $function = array_filter(self::$tools, function ($tool) use ($toolCall) {
                        return $tool['function']['name'] == $toolCall->function->name;
                    });

                    if (count($function) > 0) {
                        $function = array_values($function)[0];
                        $parameters = [];

                        $args = json_decode(($toolCall->function->arguments), true);
                        foreach ($function['function']['parameters']['properties'] as $name => $property) {
                            $parameters[] = $args[$name];
                        }

                        $result = call_user_func_array([$this, $function['function']['name']], $parameters);
                        $results[] = [
                            'tool_call_id' => $toolCall->id,
                            'role' => 'tool',
                            'name' => $function['function']['name'],
                            'content' => $result ?? 'Sorry, I couldn\'t find the function ' . $function['function']['name'] . '.',
                        ];
                    } else {
                        $results[] = [
                            'tool_call_id' => $toolCall->id,
                            'role' => 'tool',
                            'name' => $toolCall->function->name,
                            'content' => 'Sorry, I couldn\'t find the function ' . $toolCall->function->name . '.',
                        ];
                    }
                }
            }
        }

        if (count($results) > 0) {

            // Prepend $message to $results
            $append = [
                'role' => $message->role,
                'content' => $message->content,
            ];

            if ($message->toolCalls) {
                $append['tool_calls'] = $message->toolCalls;
            }

            self::$context_messages[] = $append;

            return $this->sendMessages($results, true);
        }

        return [$response];
    }


    /**
     * Nettoyage des messages
     */
    private function compressRedundantMessages()
    {
        $compressedMessages = [];

        foreach (self::$context_messages as $message) {
            // Recherchez si un message similaire existe déjà dans les messages compressés
            $lastIndex = count($compressedMessages) - 1;

            if ($lastIndex >= 0 && $compressedMessages[$lastIndex]['role'] === $message['role'] && in_array($message['role'], ['user', 'system'])) {
                // Si le message actuel a le même rôle que le dernier message compressé,
                // ajoutez son contenu au contenu du dernier message
                $compressedMessages[$lastIndex]['content'] .= "\n" . $message['content'];
            } else {
                // Sinon, ajoutez le message comme un nouveau message compressé
                $compressedMessages[] = $message;
            }
        }

        // Mettez à jour le tableau context_messages avec les messages compressés
        self::$context_messages = $compressedMessages;
    }


    /**
     * Tools
     */

    private function get_current_weather($location, $unit = 'celsius')
    {

        try {
            $url = 'https://api.openweathermap.org/data/2.5/weather?q=' . $location . '&appid=' . config('chat.weather_api_key') . '&units=' . $unit;
            $data = json_decode(file_get_contents($url), true);
            if ($data['cod'] == 200) {
                return 'The current weather in ' . $location . ' is ' . $data['weather'][0]['description'] . ' with a temperature of ' . $data['main']['temp'] . '°' . strtoupper(substr($unit, 0, 1)) . '.';
            } else {
                return 'Sorry, I couldn\'t find the weather in ' . $location . '.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t find the weather in ' . $location . '. Error: ' . $e->getMessage();
        }
    }

    private function get_location($location)
    {
        $url = 'https://api.openweathermap.org/geo/1.0/direct?q=' . urlencode($location) . '&appid=' . config('chat.weather_api_key') . '&limit=1';
        $data = json_decode(file_get_contents($url), true);

        $location = $data[0];
        if ($location) {
            return implode(', ', [
                'lat' => $location['lat'],
                'lon' => $location['lon'],
                'name' => $location['name'],
                'state' => $location['state'],
                'country' => $location['country'],
            ]);
        } else {
            return 'Sorry, I couldn\'t find the location ' . $location . '.';
        }
    }

    private function list_files()
    {

        // List files and folder recursively
        $list_files = function ($path) use (&$list_files) {
            $files = glob($path . '/*');
            $result = [];
            foreach ($files as $file) {

                // Get relative path from documents folder
                $path = str_replace(App::STORAGE_PATH . '/documents/', '', $file);

                if (is_dir($file)) {
                    $result[] = [
                        'type' => 'folder',
                        'name' => basename($file),
                        'path' => $path,
                        'children' => $list_files($file),
                    ];
                } else {
                    $result[] = [
                        'type' => 'file',
                        'name' => basename($file),
                        'path' => $path,
                    ];
                }
            }
            return $result;
        };

        $data = $list_files(App::STORAGE_PATH . '/documents');

        // Return to json
        return "Here is the list of files and folders in the documents folder (in JSON):\n" . json_encode($data, JSON_PRETTY_PRINT);
    }

    private function read_file($file)
    {
        try {
            $path = App::STORAGE_PATH . '/documents/' . $file;
            if (file_exists($path)) {
                return file_get_contents($path);
            } else {
                return 'Sorry, I couldn\'t find the file ' . $file . '.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t find the file ' . $file . '. Error: ' . $e->getMessage();
        }
    }

    private function write_file($file, $content)
    {
        try {
            // Get absolute path
            $path = App::STORAGE_PATH . '/documents/' . $file;

            // If folder not exists, create it
            if (!is_dir(dirname($path))) {
                mkdir(dirname($path), 0777, true);
            }

            // Write file
            $res = file_put_contents($path, $content);

            if ($res) {
                return 'The file ' . $file . ' has been successfully written.';
            } else {
                return 'Sorry, I couldn\'t write the file ' . $file . '.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t write the file ' . $file . '. Error: ' . $e->getMessage();
        }
    }

    private function delete_file($file)
    {
        try {
            // Get absolute path
            $path = App::STORAGE_PATH . '/documents/' . $file;

            // Delete file
            $res = unlink($path);

            if ($res) {
                return 'The file ' . $file . ' has been successfully deleted.';
            } else {
                return 'Sorry, I couldn\'t delete the file ' . $file . '.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t delete the file ' . $file . '. Error: ' . $e->getMessage();
        }
    }

    private function add_memory($keywords, $content)
    {
        try {
            $user = (!Helper::isCLI() && Auth::check()) ? Auth::user()->username : null;
            $res = Memory::add($keywords, $content, $user);

            if ($res) {
                return 'The memory data has been successfully added.';
            } else {
                return 'Sorry, I couldn\'t add the memory data.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t add the memory data. Error: ' . $e->getMessage();
        }
    }

    private function search_memory($keywords)
    {
        try {
            $res = Memory::search($keywords);

            if ($res) {
                return 'The memory data has been successfully found. Result: ' . json_encode($res);
            } else {
                return 'Sorry, I couldn\'t find the memory data.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t find the memory data. Error: ' . $e->getMessage();
        }
    }

    private function remove_memory($id)
    {
        try {
            $res = Memory::where('id', $id)->delete();

            if ($res) {
                return 'The memory data has been successfully removed.';
            } else {
                return 'Sorry, I couldn\'t remove the memory data.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t remove the memory data. Error: ' . $e->getMessage();
        }
    }

    private function get_last_activity()
    {
        $thommy = User::find(1);
        return 'The last activity of Thommy was on ' . $thommy->last_online . '.';
    }

    private function open_url($url)
    {
        try {
            $url = str_replace(' ', '%20', $url);
            $res = file_get_contents($url);
            if ($res) {
                return 'The URL ' . $url . ' has been successfully opened. Content: ' . $res;
            } else {
                return 'Sorry, I couldn\'t open the URL ' . $url . '.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t open the URL ' . $url . '. Error: ' . $e->getMessage();
        }
    }

    private function send_discord_message($user, $message)
    {
        try {
            $result = json_decode($this->make_discord_request('users/@me/channels', [
                'recipient_id' => $user,
            ]), true);

            if (isset($result['id'])) {
                $result = $this->make_discord_request('channels/' . $result['id'] . '/messages', [
                    'content' => $message,
                ]);
                return 'Result: ' . $result;
            } else {
                return 'Sorry, I couldn\'t send the message to the user ' . $user . '. Result: ' . json_encode($result);
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t send the message to the user ' . $user . '. Error: ' . $e->getMessage();
        }
    }

    private function make_discord_request($endpoint, $data = [], $method = 'POST')
    {
        try {
            $url = 'https://discord.com/api/' . $endpoint;

            $ch = curl_init($url);

            # Set headers, data etc..
            curl_setopt_array($ch, array(
                CURLOPT_HTTPHEADER     => array(
                    'Authorization: Bot ' . config('chat.discord_api_token'),
                    "Content-Type: application/json",
                    "Accept: application/json"
                ),
                CURLOPT_RETURNTRANSFER => 1,
                CURLOPT_FOLLOWLOCATION => 1,
                CURLOPT_VERBOSE        => 1,
                CURLOPT_SSL_VERIFYPEER => 0
            ));

            if ($method == 'POST') {
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
            } else if ($method == 'GET') {
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "GET");
                $url = sprintf("%s?%s", $url, http_build_query($data));
            } else if ($method == 'PUT') {
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PUT");
                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
            } else if ($method == 'DELETE') {
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "DELETE");
            } else if ($method == 'PATCH') {
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "PATCH");
                curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
            } else {
                throw new Exception('Method not allowed');
            }

            // Set url
            curl_setopt($ch, CURLOPT_URL, $url);

            # Send request
            $result = curl_exec($ch);
            $error = curl_error($ch);
            curl_close($ch);

            if ($error) {
                return 'Curl error: ' . $error;
            }

            return $result;
        } catch (Exception $e) {
            return 'Error: ' . $e->getMessage();
        }
    }

    private function exec_php($code)
    {
        try {
            ob_start();
            eval("?>" . $code . "<?php ");
            $result = ob_get_clean();

            if ($result) {
                return 'The PHP code has been successfully executed. Result: ' . json_encode($result);
            } else {
                return 'Sorry, I couldn\'t execute the PHP code.';
            }
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t execute the PHP code. Error: ' . $e->getMessage();
        }
    }

    private function get_current_user()
    {
        return json_encode(Auth::user());
    }

    private function get_user_by_id($id)
    {
        $user = User::find($id);
        if ($user) {
            return json_encode($user);
        } else {
            return 'Sorry, I couldn\'t find the user with the id ' . $id . '. Try searching information in files.';
        }
    }

    private function find_users_by_name($name)
    {
        $users = User::where('username', 'LIKE', '%' . $name . '%')->get();
        if ($users) {
            return json_encode($users);
        } else {
            return 'Sorry, I couldn\'t find the users with the name ' . $name . '. Try searching information in files.';
        }
    }

    private function list_users()
    {
        $users = User::get();
        if ($users) {
            return json_encode($users);
        } else {
            return 'Sorry, I couldn\'t list the users. Try searching information in files.';
        }
    }

    private function clear_cache()
    {
        Cache::clear();
        return 'The cache has been successfully cleared.';
    }

    private function send_mail($to, $subject, $html_message)
    {
        try {
            return "Email sent. Result: " . json_encode((new CustomEmail($to, $subject, $html_message))->send());
        } catch (Exception $e) {
            return 'Sorry, I couldn\'t send the email. Error: ' . $e->getMessage();
        }
    }
}
