<?php

use Core\Request;

/**
 * Dump and die variables
 * 
 * @param mixed $vars
 */
function dd($vars) {
    \Core\Helper::dd($vars);
}

/**
 * Dump variables
 * 
 * @param mixed $vars
 */
function dump($vars) {
    \Core\Helper::dump($vars);
}

/**
 * Get a config value
 * 
 * @param string $key
 * @param mixed $default
 * @return mixed
 */
function config($key, $default = null) {
    return \Core\Config::get($key, $default);
}

/**
 * Get a route URL by name
 * 
 * @param string $name
 * @param array $params 
 */
function route($name, $params = []) {
    return \Core\Router::route($name, $params);
}

/**
 * Get a route full URL by URI
 * 
 * @param string $uri
 * @param array $params 
 */
function url($uri, $params = []) {
    return config('app.url') . '/' . ltrim($uri, '/') . ($params ? '?' . http_build_query($params) : '');
}

/**
 * Get the asset path
 * 
 * @param string $path
 * @return string
 */
function asset($path) {
    return \Core\Router::asset($path);
}

/**
 * Check if the current route is the same as the given route
 * 
 * @param string $name
 * @return bool
 */
function isRoute($name) {
    return \Core\Router::isName($name);
}

/**
 * Check if the current URI is the same as the given URI
 * 
 * @param string $uri
 * @return bool
 */
function isUri($uri) {
    return \Core\Router::isUri($uri);
}

/**
 * Get the current route
 * 
 * @return array
 */
function currentRoute() {
    return \Core\Router::current();
}

/**
 * Get the current request
 * 
 * @return \Core\Request
 */
function request() {
    return \Core\Request::instance();
}

/**
 * Get old input value
 * 
 * @return string
 */
function old($key) {
    return \Core\Request::old($key);
}

/**
 * Get an error flash message by key and return it
 * 
 * @param string $key
 * @return string
 */
function error($key) {
    $val = \Core\Request::showFlash("errors_input.$key");
    if(!is_string($val)) {
        $val = null;
    }

    if($val) {
        $GLOBALS['FLASH']['errors_input'][$key] = $val;
    }else{
        $val = $GLOBALS['FLASH']['errors_input'][$key] ?? null;
    }

    return $val;
}

/**
 * Translate a key and return it
 * 
 * @param string $key
 * @param array $params
 */
function __($key, $params = []) {
    return \Core\Lang::get($key, $params);
}

/**
 * Get the current language code
 * 
 * @return string
 */
function lang_code() {
    return \Core\Lang::current();
}

/**
 * Get an environment value
 */
function env($key, $default = null) {
    return \Core\Env::get($key, $default);
}

/**
 * Translate a key and echo it
 * 
 * @param string $key
 * @param array $params
 */
function _e($key, $params = []) {
    echo \Core\Lang::get($key, $params);
}

/**
 * Escape a string
 * 
 * @param string $string
 */
function e($string) {
    echo htmlspecialchars($string);
}

/**
 * Render a view
 * 
 * @param string $name
 * @param array $params
 */
function view($name, $params = []) {
    return \Core\View::render($name, $params);
}

/**
 * Redirect to a URL
 * 
 * @param string $url
 */
function redirect($url) {
    return \Core\Helper::redirect($url);
}

/**
 * Get the current user
 * 
 * @return \Models\User
 */
function user() {
    return \Core\Auth::user();
}

/**
 * Return the path to the base folder
 * 
 * @param string $path
 */
function base_path($path) {
    $path = \Core\App::BASE_PATH . $path;
    $path = str_replace(["/", "\\"], DIRECTORY_SEPARATOR, $path);

    return $path;
}

/**
 * Return the path to the storage folder
 * 
 * @param string $path
 */
function storage_path($path) {
    $path = \Core\App::STORAGE_PATH . $path;
    $path = str_replace(["/", "\\"], DIRECTORY_SEPARATOR, $path);

    return $path;
}

/**
 * Return the path to the public folder
 * 
 * @param string $path
 */
function public_path($path) {
    return \Core\App::PUBLIC_PATH . $path;
}

/**
 * Forelse loop. If the array is empty, the emptyCallback will be called.
 * 
 * @param array $array
 * @param callable $callback 
 * @param callable $emptyCallback
 */
function forelse($array, $callback, $emptyCallback) {
    if (count($array)) {
        foreach ($array as $key => $value) {
            $callback($value, $key);
        }
    } else {
        $emptyCallback();
    }
}