<?php
namespace Core;

class Auth {
    
    /**
     * Return the current user
     * 
     * @return \Models\User|null
     */
    public static function user() {
        if(isset($_SESSION['user'])) {

            if(Config::get('auth.user_model', null) !== null) {
                $user_model = Config::get('auth.user_model');
                $user_model = new $user_model();
                $user = $user_model->find($_SESSION['user']['id']);
                if($user === null) {
                    self::logout();
                    return null;
                }
                
                return $user;
            }

            return new Collection($_SESSION['user']);
        }
        
        return null;
    }

    /**
     * Return the current user role
     * 
     * @return string|null
     */
    public static function role() {
        if(Config::get('auth.user_model', null) === null) {
            throw new \Exception('You must specify a user model in the auth config file');
        }

        if(Config::get('auth.role_column', null) === null) {
            throw new \Exception('You must specify a role column in the auth config file');
        }

        if(Auth::user() === null) {
            return null;
        } else {
            return Auth::user()->{Config::get('auth.role_column')};
        }
    }

    /**
     * Check if the current user has a role
     * 
     * @param string $role
     * @return bool
     */
    public static function hasRole($role) {
        if(Config::get('auth.user_model', null) === null) {
            throw new \Exception('You must specify a user model in the auth config file');
        }

        if(Config::get('auth.role_column', null) === null) {
            throw new \Exception('You must specify a role column in the auth config file');
        }

        if(Auth::user() === null) {
            return false;
        } else {
            if(Auth::role() !== $role) {
                return false;
            } else {
                return true;
            }
        }
    }

    /**
     * Check if the current user has one of the roles
     * 
     * @param array $roles
     * @return bool
     */
    public static function hasRoles(array $roles) {
        foreach($roles as $role) {
            if(self::hasRole($role)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Check if the current user is logged in
     * 
     * @return bool
     */
    public static function check() {
        return self::user() !== null;
    }

    /**
     * Logout the current user
     * 
     * @return void
     */
    public static function logout() {
        unset($_SESSION['user']);
    }

    /**
     * Login a user
     * 
     * @param array $user
     * @return void
     */
    public static function login(array $user) {
        $_SESSION['user'] = $user;
    }
}