<?php
namespace Core;

class BaseController {

    public function __construct() { }
    
    public function init() {
        // Handle middleware
        Middleware::run();
    }
    
    public function request() {
        return Request::instance();
    }

    public function middleware(string|array $middlewares) {
        if (!is_array($middlewares)) {
            Middleware::add($middlewares);
        }else{
            foreach ($middlewares as $middleware) {
                Middleware::add($middleware);
            }
        }
    }

    /**
     * Return a json response 
     * 
     * @param array $data
     * @return void
     */
    public function json($data, $response_code = 200) {
        if(!Helper::isCLI()) {
            http_response_code($response_code);
            header('Content-Type: application/json');
        }
        
        echo json_encode($data);
        exit;
    }

    /**
     * Return a view response
     * 
     * @param string $name
     * @param array $params
     * @return void
     */
    public function view(string $name, array $params = []) {
        return View::render($name, $params);
    }

    /**
     * Return a file response
     * 
     * @param string $file
     * @return void
     */
    public function file(string $file) {
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="' . basename($file) . '"');
        header('Content-Length: ' . filesize($file));
        readfile($file);
        exit;
    }

    /**
     * Redirect to a specific url
     * 
     * @param string $url
     * @return void
     */
    public function redirect($url) {
        return Helper::redirect($url);
    }

    /**
     * Redirect to the previous page
     * 
     * @return void
     */
    public function back() {
        header('Location: ' . $_SERVER['HTTP_REFERER']);
        exit;
    }

    /**
     * Auth only middleware
     */
    public function authOnly() {
        if($this->request()->user() === null) {
            $this->redirect(route('login.index'));
        } else {
            return true;
        }
    }

    /**
     * Guest only middleware
     */
    public function guestOnly() {
        if($this->request()->user() !== null) {
            $this->redirect('/');
        } else {
            return true;
        }
    }

    /**
     * Role only middleware
     * 
     * @param string $role
     * @return void
     */
    public function roleOnly($role) {
        if($this->request()->user() === null) {
            $this->redirect(route('login.index'));
        } else {
            if(Auth::hasRole($role) === false) {
                $this->redirect('/');
            } else {
                return true;
            }
        }
    }

    /**
     * Roles only middleware
     * 
     * @param array $roles
     * @return void
     */
    public function rolesOnly(array $roles) {
        if($this->request()->user() === null) {
            $this->redirect(route('login.index'));
        } else {
            if(Auth::hasRoles($roles) === false) {
                $this->redirect('/');
            } else {
                return true;
            }
        }
    }
}