<?php

namespace Core;

class BaseMiddleware
{

    public static $middlewares = [];

    public static function add($middleware)
    {

        if(!class_exists($middleware)) {
            throw new \Exception("The middleware class {$middleware} does not exist");
        }

        if(!is_subclass_of($middleware, BaseMiddleware::class)) {
            throw new \Exception("The middleware {$middleware} must be a subclass of Core\BaseMiddleware");
        }

        if(!method_exists($middleware, 'handle')) {
            throw new \Exception("The middleware {$middleware} must implement handle() method");
        }

        if(!in_array($middleware, self::$middlewares))
            self::$middlewares[] = $middleware;
    }

    public static function get()
    {
        return self::$middlewares;
    }

    public static function run()
    {
        foreach (self::$middlewares as $middleware) {
            $middleware = new $middleware();
            if($result = $middleware->handle(Request::instance()) !== $middleware->next()) {
                return $result;
            }
        }
    }

    /**
     * Handle the middleware
     * 
     * @param Request $request
     * @return void
     */
    public function handle(Request $request) {
        throw new \Exception("You must implement handle() method in your middleware");
    }


    /**
     * Call the next middleware
     * 
     * @return bool
     */
    public function next() {
        return true;
    }
}
