<?php
namespace Core;

class Cli {
    public static $singleton;
    public static $commands = [];

    public static function instance() {
        if(!self::$singleton) {
            self::$singleton = new self();
        }

        return self::$singleton;
    }

    public function __construct() {
        $this->initCommands();
    }

    private function initCommands() {
        self::$commands = glob(dirname(__FILE__) . '/../Commands/*.php');
        foreach(self::$commands as &$command) {
            $command = explode('/', $command);
            $command = end($command);
            $command = explode('.', $command);
            $command = $command[0];
            $command = '\\Commands\\' . $command;
            $command = new $command();
        }
    }

    public function run() {
        global $argv;

        if(count($argv) < 2) {
            $this->showHelp();
            return;
        }

        $command = $argv[1];
        $args = array_slice($argv, 2);
        
        foreach(self::$commands as $cmd) {
            if($cmd->name == $command) {
                $cmd->run($args);
                return;
            }
        }

        $this->showHelp();
    }

    private function showHelp() {
        echo 'Usage: php cli.php [command]' . PHP_EOL;
        echo 'Available commands:' . PHP_EOL;
       
        foreach(self::$commands as $command) {
            $command->showHelp();
        }
    }
}