<?php
namespace Core;

class Config {
    private static $config = [];
    public static $cache_enabled = null;
    public static $cache_duration = null;

    /**
     * Get a config value
     *
     * @param string $arg
     * @param mixed $default
     * @return mixed
     */
    public static function get($arg, $default = null) {

        if(self::$cache_enabled === null) {
            self::$cache_enabled = false;
            self::$cache_enabled = self::get('config.cache_enabled', false);
        }

        if(self::$cache_duration === null) {
            self::$cache_duration = 0;
            self::$cache_duration = self::get('config.cache_duration', 0);
        }


        $data = explode('.', $arg);
        $file = $data[0];
        $key = count($data) > 1 ? array_slice($data, 1) : null;

        if (!isset(self::$config[$file])) {

            // Load from cache
            if(self::$cache_enabled === true && Cache::exists('config.' . $file)) {
                self::$config[$file] = Cache::get('config.' . $file, []);
                return self::get($arg, $default);
            }

            // Load env
            Env::load();

            // Load config file
            $path = dirname(__FILE__) . '/../Configs/' . $file . '.php';
            if (is_readable($path)) {
                self::$config[$file] = require $path;
                if(self::$cache_enabled === true) {
                    Cache::set('config.' . $file, self::$config[$file], self::$cache_duration);
                }
            } else {
                return $default;
            }
        }

        if ($key === null) {
            return self::$config[$file];
        }

        $value = self::$config[$file];
        foreach ($key as $k) {
            if (!isset($value[$k])) {
                return $default;
            }
            $value = $value[$k];
        }

        return $value;
    }

    /**
     * Get all config values
     *
     * @return array
     */
    public static function getAll() {
        return self::$config;
    }
}