<?php
namespace Core;

class DB extends BaseModel {
    /**
     * Get the instance to use the ModelBuilder on a table without using a Model
     * 
     * @param string $table The table name to use
     * @return \Core\ModelBuilder
     */
    public static function table($table) {
        $model = new static();
        $model->table = $table;
        $model->timestamps = false;
        return $model;
    }

    /**
     * Execute a raw SQL query and return the result
     * 
     * @param string $sql The SQL query to execute
     * @param array $params The parameters to bind to the query
     * @return Collection|bool The result of the query
     */
    public static function execute($sql, $params = []) {
        $query = Database::getInstance()->prepare($sql);
        $request = $query->execute($params);
        
        if(substr($sql, 0, 6) == 'SELECT') {
            return new Collection($query->fetchAll(\PDO::FETCH_OBJ));
        }

        return $request;
    }
}