<?php
namespace Core;

class Env {
    public static $LOADED = false;
    
    public static function load() {
        if (self::$LOADED) {
            return;
        }

        $env = [];
        $env_file = App::BASE_PATH . '/.env';

        if (file_exists($env_file)) {
            $env_lines = file($env_file);
            foreach ($env_lines as $env_line) {
                $env_line = trim($env_line);
                if (empty($env_line) || $env_line[0] == '#') {
                    continue;
                }
                $env_line = explode('=', $env_line);
                $env_key = trim($env_line[0]);
                $env_value = trim($env_line[1]);

                // Parse value if having ${} syntax
                if (preg_match('/\${(.+)}/', $env_value, $matches)) {
                    $env_value = getenv($matches[1]);
                }

                // Parse value type
                if (preg_match('/^"(.+)"$/', $env_value, $matches)) {
                    $env_value = $matches[1]; // Remove quotes
                } else if (preg_match('/^\'(.+)\'$/', $env_value, $matches)) {
                    $env_value = $matches[1]; // Remove quotes
                } else if (preg_match('/^([0-9]+)$/', $env_value, $matches)) {
                    $env_value = (int) $matches[1]; // Convert to int
                } else if (preg_match('/^([0-9]+.[0-9]+)$/', $env_value, $matches)) {
                    $env_value = (float) $matches[1]; // Convert to float
                } else if (preg_match('/^(true|false)$/', $env_value, $matches)) {
                    $env_value = $matches[1] == 'true'; // Convert to bool
                }

                $env[$env_key] = $env_value;
            }
        }

        foreach ($env as $env_key => $env_value) {
            if(!self::has($env_key)) {
                self::set($env_key, $env_value);
            }
        }

        self::$LOADED = true;
    }

    public static function get($key, $default = null) {
        $value = getenv($key);
        if ($value === false) {
            return $default;
        }
        return $value ?: $default;
    }

    public static function set($key, $value) {
        putenv("$key=$value");
    }

    public static function has($key) {
        return getenv($key) !== false;
    }

    public static function unset($key) {
        putenv($key);
    }

    public static function clear() {
        $vars = self::vars();
        foreach ($vars as $var) {
            self::unset($var);
        }
    }

    public static function vars() {
        return array_keys($_ENV);
    }

    public static function all() {
        return $_ENV;
    }
}