<?php
namespace Core;

class Lang {
    public static $singleton = null;
    public static $translations = [];
    public static $missing_translations = [];
    public static $lang = "en";

    public static function instance() {
        if (self::$singleton === null) {
            self::$singleton = new Lang();
        }
        return self::$singleton;
    }

    public function load($lang = null) {
        self::$lang = $this->getLang();

        if($lang === null) {
            $lang = self::$lang;
        }

        $file = dirname(__FILE__) . '/../Langs/' . $lang . '.php';
        if (file_exists($file)) {
            self::$translations[$lang] = require_once $file;
        }else{
            self::$translations[$lang] = require_once dirname(__FILE__) . '/../Langs/' . Config::get('app.default_lang', 'en') . '.php';
        }
    }

    public static function current() {
        return self::$lang;
    }

    /**
     * Get the current language from cookie or default
     *
     * @return string
     */
    public function getLang() {
        $request = Request::instance();
        $lang = $request->cookie('lang');
        if ($lang === null) {
            $lang = Config::get('app.default_lang', 'en');
        }
        return $lang;
    }

    /**
     * Set the current language
     *
     * @param string $lang
     * @return void
     */
    public function setLang($lang) {
        self::$lang = $lang;
        $request = Request::instance();
        $request->setCookie('lang', $lang, time() + 60 * 60 * 24 * 30);
    }

    private function getKey($key) {

       if (isset(self::$translations[self::$lang][$key])) {
            return self::$translations[self::$lang][$key];
        }else{

            if(config('app.save_missing_translation')) {
                $this->saveMissingTranslation($key);
            }

            if ($fb = Config::get('app.fallback_lang') !== null) {
                
                if(!isset(self::$translations[$fb])) {
                    $this->load($fb);
                }

                if (isset(self::$translations[$fb][$key])) {
                    return self::$translations[$fb][$key];
                }
            }
        }

        return "translation." . $key;
    }

    /**
     * Save a missing translation key in the $lang_missing.php file
     * 
     * @param string $key
     * @return void
     */
    private function saveMissingTranslation($key) {
        // Parse the file to append the new key if exist or create it
        $file = App::SRC_PATH . '/Langs/' . self::$lang . '_missing.php';

        if(!isset(self::$missing_translations[self::$lang])) {
            if (file_exists($file)) {
                $translations = require_once $file;
                $translations = is_array($translations) ? $translations : [];
            }else{
                $translations = [];
            }

            self::$missing_translations[self::$lang] = $translations;
        }else{
            $translations = self::$missing_translations[self::$lang];
        }

        if(isset($translations[$key])) {
            return;
        }
        
        $translations[$key] = $key;
        self::$missing_translations[self::$lang][$key] = $key;
        
        $content = "<?php\nreturn [\n";
        foreach($translations as $k => $v) {
            $content .= "    '$k' => '$v',\n";
        }
        $content .= "];";

        file_put_contents($file, $content);
    }

    /**
     * Get a translation key from the current language
     *
     * @param string $key
     * @param array $params
     * @return string
     */
    public static function get($key, $params = []) {
        $instance = self::instance();
        $key = $instance->getKey($key);
        if (count($params) > 0) {
            foreach ($params as $k => $v) {
                if(empty($k) || empty($v)) continue;
                $key = str_replace(':' . $k, $v, $key);
            }
        }
        return $key;
    }
}