<?php
namespace Core;

class Logger {
    
    /**
     * Log a message
     *
     * @param string $message Message to log
     * @param string $type Type of message (debug, info, warning, error)
     * @param string $file File to log in
     * @return void
     */
    public static function log($message, $type = "INFO", $file = "app") {

        if(Helper::isCLI()) {
            $log_level = "DEBUG";
        }else{
            $log_level = Config::get('logger.log_level', 'DEBUG');
        }
        
        if(strtoupper($log_level) == 'NONE')
            return;

        $log_level = strtolower($log_level);
        $type = strtolower($type);

        $log_levels = [
            'debug' => 1,
            'info' => 2,
            'warning' => 3,
            'error' => 4
        ];

        if($log_levels[$type] < $log_levels[$log_level])
            return;

        $type = "[" . strtoupper($type) . "]";
        $date = "[" . date('Y-m-d H:i:s') . "]";

        if(Helper::isCLI()) {
            echo $type . ' ' . $message . PHP_EOL;
            return;
        }

        
        $message = $type . ' ' . $date . ' ' . $message . PHP_EOL;
        $file = App::LOGS_PATH . "$file-" . date('Ymd') . '.log';
        file_put_contents($file, $message, FILE_APPEND);
    }

    /**
     * Log an exception
     *
     * @param \Exception $exception
     * @return void
     */
    public static function logException(string|\Exception $exception) {

        if(is_string($exception))
            $exception = new \Exception($exception);
        elseif(!($exception instanceof \Exception))
            throw new \Exception('Invalid exception type');

        $message = $exception->getMessage() . ' - in file ' . $exception->getFile() . ' - on line ' . $exception->getLine();

        $previous = $exception;
        while($previous !== null) {
            $message .= PHP_EOL . PHP_EOL . $previous->getMessage() . ' - in file ' . $previous->getFile() . ' - on line ' . $previous->getLine();
            $message .= PHP_EOL . $previous->getTraceAsString();
            $previous = $previous->getPrevious();
        }

        self::error($message);
    }

    /**
     * Log a debug message
     *
     * @param string $message
     * @param string $file
     * @return void
     */
    public static function debug($message, $file = "app") {
        self::log($message, 'DEBUG', $file);
    }

    /**
     * Log an info message
     *
     * @param string $message
     * @param string $file
     * @return void
     */
    public static function info($message, $file = "app") {
        self::log($message, 'INFO', $file);
    }

    /**
     * Log a warning message
     *
     * @param string $message
     * @param string $file
     * @return void
     */
    public static function warning($message, $file = "app") {
        self::log($message, 'WARNING', $file);
    }

    /**
     * Log an error message
     *
     * @param string $message
     * @param string $file
     * @return void
     */
    public static function error($message, $file = "app") {
        self::log($message, 'ERROR', $file);
    }
}