<?php

namespace Core;
class Mail {

    /**
     * @var array
     */
    private $params = [];
    /**
     * @var MailTemplate
     */
    private $mailTemplate;

    public function __construct(MailTemplate $mailTemplate, $params = []) {
        $this->mailTemplate = $mailTemplate;

        // Set default params
        $this->params = array_merge([
            'driver' => config('mail.driver', 'smtp'), // smtp, sendmail, mail, gmail
            'host' => config('mail.host', 'localhost'),
            'port' => config('mail.port', 25),
            'username' => config('mail.username', ''),
            'password' => config('mail.password', ''),
            'from' => $mailTemplate->from ?? config('mail.from', ''),
            'from_name' => $mailTemplate->from_name ?? config('mail.from_name', ''),
            'reply_to' => $mailTemplate->replyTo ?? config('mail.reply_to', ''),
            'reply_to_name' => $mailTemplate->replyTo_name ?? config('mail.reply_to_name', ''),
            'sendmail_path' => config('mail.sendmail_path', '/usr/sbin/sendmail -bs'),
            'secure' => config('mail.secure', 'tls'),
        ], $params);

        // Add default headers
        $this->params['headers'] = [
            'MIME-Version: 1.0',
            'Content-type: text/html; charset=utf-8',
            'From: ' . $this->params['from'],
            'Reply-To: ' . $this->params['reply_to'],
        ];
    }

    public static function send(MailTemplate $mailTemplate) {
        try {
            $mail = new self($mailTemplate);
    
            if($mail->params['driver'] === 'log') {
                Logger::debug('Mail sent to ' . $mail->mailTemplate->to);
                Logger::debug('Subject: ' . $mail->mailTemplate->subject);
                Logger::debug("Body:\n" . $mail->mailTemplate->buildBody());
    
                return true;
            }
    
            $mailer = new \PHPMailer\PHPMailer\PHPMailer(true);
    
            if ($mail->params['driver'] === 'smtp') {
                $mailer->isSMTP();
                $mailer->Host = $mail->params['host'];
                $mailer->SMTPAuth = true;
                $mailer->Username = $mail->params['username'];
                $mailer->Password = $mail->params['password'];
                $mailer->SMTPSecure = $mail->params['secure'];
                $mailer->Port = $mail->params['port'];
            } elseif ($mail->params['driver'] === 'sendmail') {
                $mailer->isSendmail();
                $mailer->Sendmail = $mail->params['sendmail_path'];
            } elseif ($mail->params['driver'] === 'mail') {
                $mailer->isMail();
            } elseif ($mail->params['driver'] === 'gmail') {
                $mailer->isSMTP();
                $mailer->Host = 'smtp.gmail.com';
                $mailer->SMTPAuth = true;
                $mailer->Username = $mail->params['username'];
                $mailer->Password = $mail->params['password'];
                $mailer->SMTPSecure = 'ssl';
                $mailer->Port = 465;
            }else{
                throw new \Exception('Invalid mail driver: ' . $mail->params['driver']);
            }
            
            $mailer->setFrom($mail->params['from'], $mail->params['from_name']);
            $mailer->addReplyTo($mail->params['reply_to'], $mail->params['reply_to_name']);
            $mailer->isHTML(true);

            if(!empty($mail->mailTemplate->to))
                $mailer->addAddress($mail->mailTemplate->to);
            else throw new \Exception('Invalid mail to: ' . $mail->mailTemplate->to);

            if(!empty($mail->mailTemplate->cc))
                $mailer->addCC($mail->mailTemplate->cc);
            if(!empty($mail->mailTemplate->bcc))
                $mailer->addBCC($mail->mailTemplate->bcc);

            $mailer->Subject = $mail->mailTemplate->subject;
            $mailer->Body = $mail->mailTemplate->buildTemplate(true);
            $mailer->AltBody = $mail->mailTemplate->buildTemplate(false);
    
            return $mailer->send();
        }
        catch(\Exception $e) {
            Logger::logException($e);
            throw $e;
        }
    }

}