<?php
namespace Core;

class Pagination implements \IteratorAggregate {

    public $total;
    public $perPage;
    public $items;
    public $currentPage;
    public $lastPage;
    public $totalPage;
    public $from;
    public $to;
    public $options;


    public static function make($perPage = 15, $currentPage = null, $options = []) {
        return new static($perPage, $currentPage, $options);
    }

    public function __construct($perPage = 15, $currentPage = null, $options = []) {
        $this->setOptions($options);
        $this->perPage($perPage);
        $this->currentPage($currentPage);
    }

    public function perPage($perPage = null) {

        if($perPage === null && $this->perPage !== null) {
            return $this->perPage;
        }

        $this->perPage = (int)$perPage ?: 15;
        return $this;
    }

    public function currentPage($currentPage = null) {

        if($currentPage === null && $this->currentPage !== null) {
            return $this->currentPage;
        }

        $this->currentPage = (int)($currentPage ?: (isset($_GET[$this->options['pageName']]) ? $_GET[$this->options['pageName']] : 1));
        return $this;
    }

    public function setOptions($options) {
        $this->options = array_merge([
            'path' => isset($_SERVER['REQUEST_URI']) ? strtok($_SERVER['REQUEST_URI'], '?') : '/',
            'query' => $_GET,
            'fragment' => '',
            'pageName' => 'page',
        ], $options);

        return $this;
    }

    public function setItems(array|Collection $items) {
        $this->items = is_array($items) ? new Collection($items) : $items;
        $this->total = (int)$this->items->count();
        $this->lastPage = (int)ceil($this->total / $this->perPage);
        $this->totalPage = (int)$this->lastPage;
        $this->from = (int)(($this->currentPage - 1) * $this->perPage + 1);
        $this->to = (int)($this->from + $this->perPage - 1);
        if($this->to > $this->total) {
            $this->to = (int)$this->total;
        }

        return $this;
    }

    public function items() {
        return $this->items ?: new Collection();
    }

    public function url($page = null) {
        $page = $page ?: $this->currentPage;
        $query = $this->options['query'];
        $query[$this->options['pageName']] = $page;
        $queryString = http_build_query($query, '', '&');
        $url = $this->options['path'] . ($queryString ? '?' . $queryString : '') . ($this->options['fragment'] ? '#' . $this->options['fragment'] : '');

        return $url;
    }

    public function getIterator(): \ArrayIterator {
        return $this->items->getIterator();
    }

    public function __get($key) {

        // Keep backward compatibility with old code
        if($key === "data") {
            return $this->items;
        }
    }

    public function links($view = "partials.pagination") {

        if(View::exists($view)) {
            View::include($view, [
                'pagination' => $this
            ]);

            return;
        }


        throw new \Exception("View [$view] not found");
    }
}