<?php
namespace Core;

class Scheduler {
    public static function __callStatic($name, $arguments) {
        return call_user_func_array([Core_Scheduler::getInstance(), $name], $arguments);
    }

    public static function getInstance() {
        return Core_Scheduler::getInstance();
    }

    public function __call($name, $arguments) {
        return call_user_func_array([Core_Scheduler::getInstance(), $name], $arguments);
    }

    public function __get($name) {
        return Core_Scheduler::getInstance()->$name;
    }

    public function __set($name, $value) {
        Core_Scheduler::getInstance()->$name = $value;
    }

    public function __isset($name) {
        return isset(Core_Scheduler::getInstance()->$name);
    }

    public function __unset($name) {
        unset(Core_Scheduler::getInstance()->$name);
    }

    public function __construct() {
        Core_Scheduler::getInstance();
    }

    // Static methods
    public static function run() {
        Core_Scheduler::getInstance()->run();
    }

    public static function schedule($name, $cron, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, $cron, $controller, $action, $withoutOverlapping);
    }

    public static function daily($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * *', $controller, $action, $withoutOverlapping);
    }
    public static function dailyAt($name, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour * * *", $controller, $action, $withoutOverlapping);
    }

    public static function hourly($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 * * * *', $controller, $action, $withoutOverlapping);
    }
    public static function hourlyAt($name, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute * * * *", $controller, $action, $withoutOverlapping);
    }

    public static function everyMinute($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '* * * * *', $controller, $action, $withoutOverlapping);
    }

    public static function everyFiveMinutes($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '*/5 * * * *', $controller, $action, $withoutOverlapping);
    }

    public static function everyTenMinutes($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '*/10 * * * *', $controller, $action, $withoutOverlapping);
    }

    public static function everyThirtyMinutes($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '*/30 * * * *', $controller, $action, $withoutOverlapping);
    }

    public static function weekly($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 0', $controller, $action, $withoutOverlapping);
    }
    public static function weeklyOn($name, $day, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour * * $day", $controller, $action, $withoutOverlapping);
    }

    public static function monthly($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 1 * *', $controller, $action, $withoutOverlapping);
    }
    public static function monthlyOn($name, $day, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour $day * *", $controller, $action, $withoutOverlapping);
    }

    public static function yearly($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 1 1 *', $controller, $action, $withoutOverlapping);
    }
    public static function yearlyOn($name, $month, $day, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour $day $month *", $controller, $action, $withoutOverlapping);
    }

    public static function weekdays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 1-5', $controller, $action, $withoutOverlapping);
    }
    public static function weekdaysAt($name, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour * * 1-5", $controller, $action, $withoutOverlapping);
    }
    public static function weekends($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 0,6', $controller, $action, $withoutOverlapping);
    }
    public static function weekendsAt($name, $hour, $minute, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, "$minute $hour * * 0,6", $controller, $action, $withoutOverlapping);
    }

    public static function mondays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 1', $controller, $action, $withoutOverlapping);
    }
    public static function tuesdays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 2', $controller, $action, $withoutOverlapping);
    }
    public static function wednesdays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 3', $controller, $action, $withoutOverlapping);
    }
    public static function thursdays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 4', $controller, $action, $withoutOverlapping);
    }
    public static function fridays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 5', $controller, $action, $withoutOverlapping);
    }
    public static function saturdays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 6', $controller, $action, $withoutOverlapping);
    }
    public static function sundays($name, $controller, $action = null, $withoutOverlapping = false) {
        Core_Scheduler::getInstance()->schedule($name, '0 0 * * 0', $controller, $action, $withoutOverlapping);
    }
}

// Cron job scheduler
class Core_Scheduler {
    public static $singleton;
    private $schedules = [];

    public static function getInstance() {
        if(self::$singleton === null)
            self::$singleton = new self();

        return self::$singleton;
    }

    public function __construct() {
        self::$singleton = $this;
        $this->init();
    }

    public static function __callStatic($name, $arguments) {
        return call_user_func_array([self::$singleton, $name], $arguments);
    }

    private function init() {
        $this->initSchedules();
    }

    private function initSchedules() {
        require_once dirname(__FILE__) . '/../schedules.php';
    }

    /**
     * Add a schedule
     *
     * @param string $name Name of the schedule
     * @param string $cron Cron expression
     * @param string $controller Controller to call
     * @param string $action Action to call
     * @param boolean $withoutOverlapping If true, the schedule will be run only once at same time
     */
    public function schedule($name, $cron, $controller, $action = null, $withoutOverlapping = false) {
        $controller = $this->initController($controller);
        $this->schedules[] = [
            'name' => $name,
            'cron' => $cron,
            'controller' => $controller,
            'action' => $action,
            'withoutOverlapping' => $withoutOverlapping
        ];
    }

    public function initController($controller) {
        if(is_callable($controller))
            return $controller;
        
        if(is_string($controller)) {
            $controller = 'Controllers\\' . $controller;

            try {
                if(class_exists($controller))
                    return new $controller();
            }
            catch(\Exception $e) {
                throw new \Exception("Controller not found: $controller");
            }
        }

        throw new \Exception("Controller not found: $controller"); 
    }

    public function run() {
        $this->runSchedules();
    }

    private function runSchedules() {
        foreach($this->schedules as $schedule) {
            if($this->isTimeToRun($schedule['cron'])) {
                $this->runSchedule($schedule);
            }
        }
    }

    private function isTimeToRun($cron) {
        $cron = explode(' ', $cron);
        $cron = array_map(function($item) {
            return explode(',', $item);
        }, $cron);
        $cron = array_map(function($item) {
            return array_map(function($item) {
                return explode('/', $item);
            }, $item);
        }, $cron);
        $cron = array_map(function($item) {
            return array_map(function($item) {
                return array_map(function($item) {
                    return explode('-', $item);
                }, $item);
            }, $item);
        }, $cron);

        $cron = [
            'minutes' => $cron[0][0][0][0],
            'hours' => $cron[1][0][0][0],
            'days' => $cron[2][0][0][0],
            'months' => $cron[3][0][0][0],
            'weekdays' => $cron[4][0][0][0]
        ];

        $cron['minutes'] = $this->isTimeToRunCronItem($cron['minutes'], date('i'));
        $cron['hours'] = $this->isTimeToRunCronItem($cron['hours'], date('H'));
        $cron['days'] = $this->isTimeToRunCronItem($cron['days'], date('d'));
        $cron['months'] = $this->isTimeToRunCronItem($cron['months'], date('m'));
        $cron['weekdays'] = $this->isTimeToRunCronItem($cron['weekdays'], date('w'));

        return $cron['minutes'] && $cron['hours'] && $cron['days'] && $cron['months'] && $cron['weekdays'];
    }

    private function isTimeToRunCronItem($cronItem, $dateItem) {
        if($cronItem === '*') {
            return true;
        }

        if(strpos($cronItem, '/') !== false) {
            $cronItem = explode('/', $cronItem);
            $cronItem = $cronItem[1];
            return $dateItem % $cronItem === 0;
        }

        if(strpos($cronItem, '-') !== false) {
            $cronItem = explode('-', $cronItem);
            $cronItem = [
                'min' => $cronItem[0],
                'max' => $cronItem[1]
            ];
            return $dateItem >= $cronItem['min'] && $dateItem <= $cronItem['max'];
        }

        if(strpos($cronItem, ',') !== false) {
            $cronItem = explode(',', $cronItem);
            return in_array($dateItem, $cronItem);
        }

        return $dateItem === $cronItem;
    }

    private function runSchedule($schedule) {

        if($schedule['withoutOverlapping'] === true) {
            if(Cache::get('scheduler_' . md5($schedule['name']), null) !== null) {
                Logger::debug('Schedule already running without overlapping. Skipping: ' . $schedule['name']);
                return;
            }
        }


        Cache::set('scheduler_' . md5($schedule['name']), time());
        Logger::log('Running schedule: ' . $schedule['name'], 'DEBUG');

        try {
            $controller = $schedule['controller'];
    
            if(is_callable($controller)) {
                $controller();
            }else{
                $action = $schedule['action'];
                $controller->$action();
            }
        }
        catch(\Exception $e) {
            Logger::error('Error running schedule: ' . $schedule['name'] . ' - ' . $e->getMessage());
        }


        Cache::delete('scheduler_' . md5($schedule['name']));
        Logger::log('Schedule finished: ' . $schedule['name'], 'DEBUG');
    }

}