<?php
namespace Core;

class View {

    public static $pushs = [];
    public static $containers = [];
    public static $layout = null;
    public static $hasNewLayout = false;
    
    /**
     * Render a view
     *
     * @param string $view View to render
     * @param array $args Arguments to pass to the view
     */
    public static function render($view, $args = []) {
        try {
            ob_start();
            self::doRender($view, $args);
            echo ob_get_clean();
        }
        catch(\Throwable $th) {
            ob_end_clean();

            if(Request::instance()->ajax()) {
                throw new \Exception("Error while rendering view $view : " . $th->getMessage(), 0, $th);
            }

            Request::instance()->flash('exception', $th);
            // Reload page
            redirect('Location: ' . Router::current()['raw']['uri']);
        }
    }
    public static function mail($view, $args = []) {
        try {
            ob_start();
            self::doRender($view, $args);
            return ob_get_clean();
        }
        catch(\Throwable $th) {
            ob_end_clean();
            throw new \Exception("Error while rendering view $view : " . $th->getMessage(), 0, $th);
        }
    }
    private static function doRender($view, $args = []) {
        $view = str_replace('.', '/', $view);
        extract($args, EXTR_SKIP);
        $file = dirname(__FILE__) . '/../Views/' . $view . '.php';
        
        if (is_readable($file)) {
            require_once $file;

            if(self::$layout !== null && self::$hasNewLayout) {
                self::$hasNewLayout = false;
                
                foreach(self::$layout as $layout) {
                    $layout = dirname(__FILE__) . '/../Views/' . $layout . '.php';
                    if(is_readable($layout)) {
                        require_once $layout;
                        self::doRender($view, $args);
                    } else {
                        throw new \Exception("Defined layout $layout not found");
                    }
                }

            }
        } else {
            throw new \Exception("View file $file not found");
        }
    }

    /**
     * Define a layout
     *
     * @param string $layout Layout to use
     * @return void
     */
    public static function layout($layout) {

        if(self::$layout === null) {
            self::$layout = [];
        }

        $layout = str_replace('.', '/', $layout);

        if(!in_array($layout, self::$layout)) {
            self::$layout[] = $layout;
            self::$hasNewLayout = true;
        }
    }

    private static function push($container, $content) {
        if(!isset(self::$pushs[$container])) {
            self::$pushs[$container] = [];
        }
        self::$pushs[$container][] = $content;
    }

    /**
     * Start a container
     *
     * @param string $container Container name
     * @return void
     */
    public static function start($container) {
        ob_start();
        self::$containers[] = $container;
    }

    /**
     * End the last container started
     *
     * @return void
     */
    public static function end() {
        $container = array_pop(self::$containers);
        self::push($container, ob_get_clean());
    }

    /**
     * Set a container content
     *
     * @param string $container Container name
     * @param string $content Content to set
     * @return void
     */
    public static function set($container, $content) {
        // Only once
        if(!isset(self::$pushs[$container])) {
            self::$pushs[$container] = [];
        }

        if(!isset(self::$pushs[$container][0])) {
            self::$pushs[$container][0] = $content;
        }
    }

    /**
     * Yield a container
     *
     * @param string $container Container name
     * @return void
     */
    public static function yield($container) {
        if(isset(self::$pushs[$container])) {
            foreach(self::$pushs[$container] as $content) {
                echo $content;
            }
        }
    }

    /**
     * Include a view
     *
     * @param string $view View to include
     * @param array $args Arguments to pass to the view
     * @return void
     */
    public static function include($view, $args = []) {
        $view = str_replace('.', '/', $view);
        extract($args, EXTR_SKIP);
        $file = dirname(__FILE__) . '/../Views/' . $view . '.php';
        if (is_readable($file)) {
            require $file;
        } else {
            throw new \Exception("View file $file not found");
        }
    }

    /**
     * Check if a view exists
     *
     * @param string $view View to check
     * @return boolean
     */
    public static function exists($view) {
        $view = str_replace('.', '/', $view);
        $file = dirname(__FILE__) . '/../Views/' . $view . '.php';
        return is_readable($file);
    }
}