<?php

use Core\View;

// On définit le layout à utiliser
View::layout("layouts.master");
// On définit le titre de la page (afficher dans le layout à l'emplacement "title")
View::set('title',  __('home'));
?>

<!-- On définit le contenu de la page (afficher dans le layout à l'emplacement "content") -->
<?php View::start("main"); ?>

<div class="text-center">
    <h1 class="text-3xl mb-2"><?= __('home'); ?></h1>
    <p class="text-gray-500 mb-2"><?= __('home_explanation'); ?></p>
</div>

<!-- Chat window: with a container with max height, and input to send message -->
<div class="flex flex-col gap-2 mt-4 mb-4 p-4 border border-gray-300 rounded-md shadow-md bg-white">
    <div class="max-h-96 overflow-y-auto">
        <div id="chat" class="flex flex-col gap-2 min-h-screen align-start justify-end">
            <p id="chat-template" class="chat-item text-gray-600 text-sm mb-2 bg-gray-100 p-2 rounded-md">
                <span class="chat-username text-gray-900 font-bold mr-2 bg-gray-300 p-1 rounded-md">
                    Assistant
                </span>

                <span class="chat-content">
                    Bienvenue sur <?= config('app.name', 'App'); ?> ! Pour commencer, dites "Bonjour" ou "Salut" !
                </span>
            </p>
        </div>
    </div>
    <!-- Loader let me think (grey light, small, italic) -->
    <div id="chat-loader" class="flex flex-col items-start justify-center gap-2 text-gray-400 hidden">
        <p class="text-sm italic">
            <!-- Loading icon -->
            <svg class="animate-spin h-5 w-5 inline-block" viewBox="0 0 24 24">
                <circle class="opacity-25" cx="12" cy="12" r="10" stroke="currentColor" stroke-width="4"></circle>
                <path class="opacity-75" fill="currentColor" d="M4 12a8 8 0 018-8V0C5.373 0 0 5.373 0 12h4z"></path>
            </svg>
            <?= __('chat_loading'); ?>
        </p>
    </div>
    <form id="chat-form" class="flex gap-2">
        <!-- Input -->
        <input id="chat-input" type="text" class="flex-1 border border-gray-300 rounded-md p-2" placeholder="<?= __('chat_placeholder'); ?>">
        <button id="chat-submit" type="submit" class="bg-blue-500 hover:bg-blue-700 text-white font-bold py-2 px-4 rounded"><?= __('chat_send'); ?></button>
    </form>
</div>

<?php View::end(); ?>

<?php View::start("scripts"); ?>
<script>
    document.addEventListener('DOMContentLoaded', function() {
        // On récupère les éléments du DOM
        const chat = document.querySelector('#chat');
        const chatTemplate = document.querySelector('#chat-template');
        const chatForm = document.querySelector('#chat-form');
        const chatInput = document.querySelector('#chat-input');
        const chatSubmit = document.querySelector('#chat-submit');
        const chatLoader = document.querySelector('#chat-loader');

        // Scroll vers le bas
        chat.parentElement.scrollTo({
            top: chat.scrollHeight,
            behavior: 'smooth'
        });

        // On définit la fonction qui ajoute un message au chat
        const addMessage = (username, content, id = null) => {

            // On vérifie si le message existe déjà
            if (id) {
                const message = chat.querySelector(`[data-message-id="${id}"]`);
                if (message) {
                    // On ajoute le message au dernier message
                    message.querySelector('.chat-content').innerHTML += content.replace(/\n/g, '<br>');
                    // On scroll vers le bas smooth
                    chat.parentElement.scrollTo({
                        top: chat.scrollHeight,
                        behavior: 'smooth'
                    });
                    // On retourne le dernier message
                    return message;
                }
            }

            // Si le dernier message appartient au même rôle, on ne crée pas de nouveau message
            if (chat.lastElementChild && chat.lastElementChild.querySelector('.chat-username').textContent === username) {
                // On ajoute le message au dernier message
                chat.lastElementChild.querySelector('.chat-content').innerHTML += '<br><br>' + content.replace(/\n/g, '<br>');
                // On scroll vers le bas smooth
                chat.parentElement.scrollTo({
                    top: chat.scrollHeight,
                    behavior: 'smooth'
                });
                // On retourne le dernier message
                return chat.lastElementChild;
            }

            // On clone le template
            const clone = chatTemplate.cloneNode(true);

            // On supprime l'id du template
            clone.removeAttribute('id');

            // On ajoute l'id du message
            if (id) {
                clone.setAttribute('data-message-id', id);
            }
            // On grise le message (en attendant la réponse)
            clone.classList.add('opacity-50');

            // On gére le multi-ligne
            if (content)
                content = content.replace(/\n/g, '<br>');

            // On ajoute le message
            clone.querySelector('.chat-username').textContent = username;
            clone.querySelector('.chat-content').innerHTML = content;

            // On ajoute le message au chat
            chat.appendChild(clone);

            // On scroll vers le bas smooth
            chat.parentElement.scrollTo({
                top: chat.scrollHeight,
                behavior: 'smooth'
            });

            // On retourne le message
            return clone;
        };

        // On définit la fonction qui affiche une erreur
        const error = (message) => {
            if (!message) {
                message = "Une erreur est survenue";
            }

            // On affiche l'erreur en dessous du dernier message
            const clone = addMessage('<?= __('system'); ?>', message);
            clone.classList.add('text-red-500');
            clone.classList.remove('opacity-50');

            // Remove loader
            loader(false);
        };

        // Affiche "Laisse moi réfléchir..." en dessus de l'input
        const loader = (bool) => {
            if (bool) {
                chatLoader.classList.remove('hidden');
                chatInput.setAttribute('disabled', 'disabled');
                chatInput.classList.add('opacity-50');
                chatSubmit.setAttribute('disabled', 'disabled');
                chatSubmit.classList.add('opacity-50');
            } else {
                chatLoader.classList.add('hidden');
                chatInput.removeAttribute('disabled');
                chatInput.classList.remove('opacity-50');
                chatSubmit.removeAttribute('disabled');
                chatSubmit.classList.remove('opacity-50');
            }
        }

        // On définit la fonction qui envoie un message
        const sendMessage = (message) => {
            // On affiche "Laisse moi réfléchir..."
            loader(true);

            // On ajoute le message au chat
            const cloneMessage = addMessage('<?= __('you'); ?>', message);
            const data = new URLSearchParams({
                message: message
            }).toString();

            // On envoie la requête
            let currentCloneMessage = null;
            fetch('<?= route('chat.store'); ?>', {
                    method: 'POST',
                    headers: {
                        'Content-Type': 'application/x-www-form-urlencoded',
                        'X-Requested-With': 'XMLHttpRequest'
                    },
                    body: data
                })
                .then(response => {
                    // On supprime la classe grisé
                    cloneMessage.classList.remove('opacity-50');

                    return response.body;
                })
                .then(rb => {
                    const reader = rb.getReader();

                    return new ReadableStream({
                        start(controller) {
                            // The following function handles each data chunk
                            function push() {
                                // "done" is a Boolean and value a "Uint8Array"
                                reader.read().then(({
                                    done,
                                    value
                                }) => {

                                    // If there is no more data to read
                                    if (done) {
                                        console.log("done", done);
                                        controller.close();

                                        // On retire le grisage
                                        if (currentCloneMessage)
                                            currentCloneMessage.classList.remove('opacity-50');

                                        // On scroll vers le bas smooth
                                        chat.parentElement.scrollTo({
                                            top: chat.scrollHeight,
                                            behavior: 'smooth'
                                        });

                                        // On vide le message
                                        currentCloneMessage = null;

                                        // On retire "Laisse moi réfléchir..."
                                        loader(false);

                                        return;
                                    }

                                    // Get the data and send it to the browser via the controller
                                    controller.enqueue(value);

                                    // Decoding the Uint8Array to UTF-8
                                    const textDecoder = new TextDecoder();

                                    // Can have multiple messages in one chunk
                                    // So need to split them using delimiter (newline)
                                    // Then parse each message

                                    // Splitting the chunk into messages
                                    const json_responses = (textDecoder.decode(value)).split('::end::');

                                    // Parsing each message
                                    for (let jsonString of json_responses) {

                                        // If empty string, skip
                                        if (!jsonString || jsonString === '') {
                                            continue;
                                        }

                                        // Parsing the JSON string
                                        let data = null;
                                        
                                        try {
                                            data = JSON.parse(jsonString);
                                            // console.log("data", data);
                                        } catch (err) {
                                            console.log("JSON.parse ERROR. Decoded value: ", jsonString);
                                            console.error(err);
                                            return;
                                        }

                                        // On verifie si la requête a échoué
                                        if (!data.success) {
                                            // On affiche l'erreur
                                            error(data?.message || "Une erreur est survenue");
                                            // On met le message en rouge
                                            cloneMessage.classList.add('text-red-500');
                                            // Ré-insert the last user message in the input
                                            chatInput.value = message;
                                            return;
                                        }

                                        // On verifie si la requête a réussi
                                        if (data.success) {

                                            // On verifie si on a un message
                                            if (!data.result) {
                                                return;
                                            }

                                            // On ajoute le message au chat
                                            const messageId = data.id;
                                            data.result.forEach((r) => {
                                                r.choices.forEach((data) => {
                                                    if (data.delta.content && data.delta.content) {
                                                        currentCloneMessage = addMessage(data.delta.role, data.delta.content, messageId);
                                                    }
                                                });
                                            });
                                        }
                                    }


                                    push();
                                });
                            }

                            // Start the reading process
                            push();
                        },
                    });
                })
                .then((stream) =>
                    // Respond with our stream
                    new Response(stream, {
                        headers: {
                            "Content-Type": "text/html"
                        }
                    }).text(),
                )
                .then((result) => {
                    // console.log("end", result);
                })
                .catch(err => {

                    // On met le message en rouge
                    cloneMessage.classList.add('text-red-500');
                    // Ré-insert the last user message in the input
                    chatInput.value = message;
                    // On retire "Laisse moi réfléchir..."
                    loader(false);

                    // On affiche l'erreur
                    error(err?.message || err || "Une erreur est survenue");
                });
        };

        // On définit la fonction qui gère l'envoi du formulaire
        const handleFormSubmit = (event) => {
            // On empêche le comportement par défaut
            event.preventDefault();
            // On récupère le message
            const message = chatInput.value;
            // On envoie le message
            sendMessage(message);
            // On vide le champ
            chatInput.value = '';
        };

        // On ajoute l'écouteur d'évènement sur le formulaire
        chatForm.addEventListener('submit', handleFormSubmit);
    });
</script>
<?php View::end(); ?>