<?php

use Core\Auth;
use Core\View;
use Core\Config;
use Core\Lang;
use Core\Request;
?>

<!DOCTYPE html>
<html lang="<?= Lang::current(); ?>">

    <head>
        <!-- Meta -->
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <meta name="description" content="<?= Lang::get('website.description') ?>">
        <meta name="keywords" content="<?= Lang::get('website.keywords') ?>">
        <meta name="author" content="">
        <meta name="robots" content="index, follow">

        <!-- Favicon -->
        <!-- <link rel="icon" href="<?= asset('images/logo.png'); ?>" type="image/png"> -->

        <!-- Title -->
        <title><?= Config::get('app.name', 'App'); ?> - <?= View::yield('title'); ?></title>

        <!-- CSS: tailwindcss -->
        <script src="https://cdn.tailwindcss.com"></script>

        <?= View::yield('head'); ?>
    </head>

    <body>


        <!-- Header -->
        <header>
            <!-- Contient les View::start('header') ... View::end() -->
            <?= View::yield("header"); ?>

            <!-- Success alert -->
            <?php if($message = Request::showFlash('success')): ?>
                <div class="bg-green-100 border border-green-400 text-green-700 px-4 py-3 rounded relative" role="alert">
                    <?= $message; ?>
                    <button type="button" class="absolute top-0 bottom-0 right-0 px-4 py-3" data-bs-dismiss="alert" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            <?php endif; ?>
            
            <!-- Error alert -->
            <?php if($message = Request::showFlash('error')): ?>
                <div class="bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative" role="alert">
                    <?= $message; ?>
                    <button type="button" class="absolute top-0 bottom-0 right-0 px-4 py-3" data-bs-dismiss="alert" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            <?php endif; ?>

            <!-- Errors alert -->
            <?php if($errors = Request::showFlash('errors')): ?>
                <div class="bg-red-100 border border-red-400 text-red-700 px-4 py-3 rounded relative" role="alert">
                    <ul>
                        <?php foreach($errors as $error): ?>
                            <li><?= $error; ?></li>
                        <?php endforeach; ?>
                    </ul>
                    <button type="button" class="absolute top-0 bottom-0 right-0 px-4 py-3" data-bs-dismiss="alert" aria-label="Close">
                        <span aria-hidden="true">&times;</span>
                    </button>
                </div>
            <?php endif; ?>
        </header>


        <!-- Contenu principal - tailwind style -->
        <main class="max-w-7xl mx-auto py-6 sm:px-6 lg:px-8">

            <?php if(Auth::check()): ?>
            <!-- Menu - with links to home and logout-->
            <nav class="flex items-center justify-between flex-wrap bg-gray-800 p-6">
                <div class="flex items-center flex-shrink-0 text-white mr-6">
                    <a href="<?= route('home'); ?>" class="font-semibold text-xl tracking-tight"><?= Config::get('app.name', 'App'); ?></a>
                </div>
                <div class="block lg:hidden">
                    <button class="flex items-center px-3 py-2 border rounded text-gray-500 border-gray-600 hover:text-white hover:border-white">
                        <svg class="fill-current h-3 w-3" viewBox="0 0 20 20" xmlns="http://www.w3.org/2000/svg">
                            <title><?= __('menu'); ?></title>
                            <path d="M0 0h20v20H0z" fill="none"/>
                            <path d="M0 5h20v2H0zM0 11h20v2H0zM0 17h20v2H0z"/>
                        </svg>
                    </button>
                </div>
                <div class="w-full block flex-grow lg:flex lg:items-center lg:w-auto">
                    <div class="text-sm lg:flex-grow">
                        <a href="<?= route('welcome') ?>" class="block mt-4 lg:inline-block lg:mt-0 text-gray-400 hover:text-white mr-4">
                            <?= __('home'); ?>
                        </a>
                        <a href="<?= route('home') ?>" class="block mt-4 lg:inline-block lg:mt-0 text-gray-400 hover:text-white mr-4">
                            <?= __('assistant'); ?>
                        </a>
                    </div>
                    <div>
                        <a href="<?= route('logout'); ?>" class="inline-block text-sm px-4 py-2 leading-none border rounded text-white border-white hover:border-transparent hover:text-gray-500 hover:bg-white mt-4 lg:mt-0"><?= __('logout'); ?></a>
                    </div>
                </div>
            </nav>
            <?php endif; ?>

            <!-- Contient les View::start('main') ... View::end() -->
            <?= View::yield("main"); ?>
        </main>


        <!-- Footer -->
        <footer class="text-center text-gray-500 text-xs mt-10 mb-5">
            <p>&copy; 2023/<?= date('Y'); ?> - <?= Config::get('app.name', 'App'); ?> - bucaille-thommy.fr</p>
        </footer>


        <!-- JS -->
        <!-- Font Awesome -->
        <!-- Documentation: https://fontawesome.com/v5.15/how-to-use/on-the-web/referencing-icons/basic-use -->
        <script src="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.4/js/all.min.js"></script>

        <!-- Contient les View::start('scripts') ... View::end() -->
        <?= View::yield("scripts"); ?>
    </body>

</html>