<?php

use Controllers\ChatController;
use Core\App;
use Core\Cache;
use Core\Scheduler;
use Models\User;

// Scheduler::schedule('test', '0 0 * * *', 'HomeController', 'index'); // Run every day at midnight
// Scheduler::schedule('test', '15 * * * *', 'HomeController', 'index'); // Run every hour at 15 minutes


// Clean logs
Scheduler::dailyAt('cleanLogs7days', '00', '10', function() {
    $files = glob(App::LOGS_PATH . '/*.log');
    foreach($files as $file) {
        if(time() - filemtime($file) >= 7 * 24 * 60 * 60) {
            @unlink($file);
        }
    }
}, null, true);

// Check presence
Scheduler::dailyAt('check_thommy_presence', '18', '00', function() {
    $thommy = User::find(1);
    if(strtotime($thommy->last_online) < strtotime('-1 day')) {
        // Is offline. Ask the chatBot what to do
        $controller = new ChatController();
        $controller->handle("Thommy semble être absent depuis +24H (depuis {$thommy->last_online}). Agit en conséquence, prévient les personnes importantes, etc.", "system");
    }
});

// Clear users chat history
Scheduler::dailyAt('clear_users_chat_history', '00', '00', function() {
    $users = User::all();
    foreach($users as $user) {
        Cache::forget("chat_messages_{$user->id}");
    }
});

// Say anything
Scheduler::dailyAt('say_anything', '09', '00', function() {
    $controller = new ChatController();
    $controller->handle("Il est 09h00. Ceci est une tâche cron. Tu es libre de l'action que tu souhaite. Regarde si tu souhaite réorganiser t'es fichiers, noter ou vérifier des choses, envoyer un coucou à Thommy ou Johnny sur discord/email... Soit intélligent et proactif. Tu as droit à une tâche auto par jour. Sert toi de ta mémoire dans t'es fichiers.", "system");
});